module.exports = function(chassis) {
  if(typeof chassis.channel !== 'function') {
    // If no real-time functionality has been configured just return
    return;
  }

  chassis.on('connection', connection => {
    // On a new real-time connection, add it to the
    // anonymous channel
    chassis.channel('anonymous').join(connection);
  });

  chassis.on('login', (user, { connection }) => {
    // connection can be undefined if there is no
    // real-time connection, e.g. when logging in via REST
    if(connection) {
      // The connection is no longer anonymous, remove it
      chassis.channel('anonymous').leave(connection);

      // Add it to the authenticated user channel
      chassis.channel('authenticated').join(connection);

      // Channels can be named anything and joined on any condition
      // E.g. to send real-time events only to admins use

      // if(user.isAdmin) { chassis.channel('admins').join(conneciton); }

      // If the user has joined e.g. chat rooms

      // user.rooms.forEach(room => chassis.channel(`rooms/${room.id}`).join(channel))
    }
  });

  chassis.publish((data, hook) => { // eslint-disable-line no-unused-vars
    // Here you can add event publishers to channels set up in `channels.js`
    // To publish only for a specific event use `chassis.publish(eventname, () => {})`

    // e.g. to publish all service events to all authenticated users use
    // return chassis.channel('authenticated');
  });

  // Here you can also add service specific event publishers
  // e..g the publish the `users` service `created` event to the `admins` channel
  // chassis.service('users').publish('created', () => chassis.channel('admins'));
};
