process.env.SUPPRESS_NO_CONFIG_WARNING = 'y';

const makeDebug = require('debug');
const path      = require('path');
const debug     = makeDebug('@feathersjs/configuration');
const config    = require('config');
const separator = path.sep;
const defaults  = require('../defaults/default.configs');

function init () {
  return function () {
    let app = this;

    const convert = current => {
      const result = Array.isArray(current) ? [] : {};

      Object.keys(current).forEach(name => {
        let value = current[name];

        if (typeof value === 'object' && value !== null) {
          value = convert(value);
        }

        if (typeof value === 'string') {
          if (value.indexOf('\\') === 0) {
            value = value.replace('\\', '');
          } else {
            if (process.env[value]) {
              value = process.env[value];
            } else if (value.indexOf('.') === 0 || value.indexOf('..') === 0) {
              // Make relative paths absolute
              value = path.resolve(
                path.join(config.util.getEnv('NODE_CONFIG_DIR')),
                value.replace(/\//g, separator)
              );
            }
          }
        }

        result[name] = value;
      });

      return result;
    };

    const env       = config.util.getEnv('NODE_ENV');
    const conf      = convert(config);
    const initConf  = app.get('init-config') || {};

    const mergedConfigs = config.util.extendDeep(defaults, initConf, conf);

    debug(`Initializing configuration for ${env} environment`);

    if (!app || app === global) {
      return mergedConfigs;
    }

    Object.keys(mergedConfigs).forEach(name => {
      let value = mergedConfigs[name];
      debug(`Setting ${name} configuration value to`, value);
      app.set(name, value);
    });
  };
}

module.exports = init;
module.exports.default = init;
