const compress      = require('compression');
const cors          = require('cors');
const helmet        = require('helmet');
// const winston       = require('winston');
const feathers      = require('@feathersjs/feathers');
const express       = require('./chassis.express');
const socketio      = require('@feathersjs/socketio');
const auth          = require('./chassis.authentication');
const jwt           = require('express-jwt');
const authAdv       = require('feathers-authentication-management');
const configuration = require('./chassis.configuration');
const middleware    = require('./chassis.middleware');
const chassisHooks  = require('./chassis.hooks');
const information   = require('./chassis.info');
const fields        = require('./info/fields');
const mongoose      = require('../persist/mongoose');
const services      = require('../services');
const channels      = require('../channels');
const docs          = require('feathers-swagger');

const processFields = require('../utils/processFields');
const defaultLogger = require('../defaults/default.logger');
const defaultSchema = require('../defaults/default.schema');

const createChassis = (nm, mdl, cnf) => {
  const conf      = cnf || {};
  const name      = nm.replace(/\s+/g, '-').toLowerCase();
  const logger    = conf.logger || defaultLogger;
  const hooks     = conf.hooks || null;
  const chassis   = express(feathers());
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // Handle Arguments //////// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  if (name)   { chassis.set('name', name); }
  if (cnf)    { chassis.set('init-config', cnf); }
  if (hooks)  { chassis.set('service-hooks', hooks); }
  if (logger) { chassis.set('service-logger', defaultLogger); }

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // Handle Model & Schema /// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  const testFields          = chassis.get('service-fields') //'*title&min|10&max|80&as|test, teaser, *text, section, image, dateFrom, dateTo, date, @!-sku'
  const envModel            = processFields(testFields)
  const initModel           = mdl || {}
  const serviceModel        = Object.assign({}, initModel, envModel)
  const rModel              = (Object.keys(serviceModel).length > 0 ? serviceModel : defaultSchema)

  chassis.set('service-model', rModel);

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // Handle configuration //// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  chassis.configure(configuration());
  // set type [authenticator, consumer]
  // const TYPE = chassis.get('type');
  // if ((TYPE[0] === 'a' || TYPE[0] === 'A')) { chassis.set('isAuth', true); }
  // if (chassis.get('public')) { chassis.set('isPublic', true); }
  // if (chassis.get('cors')) { chassis.use(cors()); }
  // if (chassis.get('secure')) { chassis.use(helmet()); }
  // if (chassis.get('compression')) { chassis.use(compress()); }

  // chassis.use(express.json());
  // chassis.use(express.urlencoded({ extended: true }));

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // REST and Sockets //////// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  chassis.configure(express.rest());

  if (chassis.get('sockets')) {
    chassis.configure(socketio());
  }

  chassis.configure(mongoose);
  chassis.configure(middleware);

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // REST and Sockets //////// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  chassis.configure(express.rest());

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // Info //////////////////// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  chassis.configure(information);
  chassis.configure(fields);

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // DOCs //////////////////// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////

  // if (chassis.get('docs')) {
  //   chassis.configure(docs({
  //     docsPath: chassis.get('docsEndpoint'),
  //     info: {
  //       title: 'A test',
  //       description: 'A description'
  //     }
  //   }));
  // }

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // authentication ////////// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // if (chassis.get('isAuth')) {
  //   if (chassis.get('authentication').secret) {
  //     chassis.configure(auth);
  //     chassis.configure(authAdv({ service: '/'+ chassis.get('name'), path: '/authenticator' }));
  //   } else {
  //     console.log("can't configure authentication.  No JWT key provided")
  //   }
  // } else {
  //   if (!chassis.get('isPublic')) {
  //     if (chassis.get('authentication').secret) {
  //       chassis.use(jwt({ secret: chassis.get('authentication').secret}));
  //     } else {
  //       console.log("can't configure authentication.  No JWT key provided")
  //     }
  //   }
  // }

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // Root Service Routing //// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  if (chassis.get('root')) {
    chassis.get('/favicon.ico', function(req, res) { res.status(204); });
    chassis.use('/', function (req, res, next) {
      req.url = '/'+chassis.get('name') + req.url;
      next();
    });
  }

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // Service ///////////////// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  chassis.configure(services);
  chassis.configure(channels);

  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  // 404 and Errors ////////// ////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////// ////////////////////////////////////////////////////////////////////////////////////
  if (chassis.get('notfound')) { chassis.use(express.notFound()); }
  if (chassis.get('errors')) { chassis.use(express.errorHandler({ logger })); }

  chassis.hooks(chassisHooks);
  return chassis;
};

module.exports = createChassis;
