const defaults = require('../defaults/default.configs')

module.exports = {
  host: {
    group         : 'system',
    type          : 'string',
    description   : 'Host where this application should be running',
    default       : defaults.host
  },

  port: {
    group         : 'system',
    type          : 'number',
    description   : 'the port on which this service will run',
    default       : defaults.port
  },

  root: {
    group: 'service',
    type: 'boolean',
    description: 'run this service as the root path for this host',
    default: defaults.root
  },

  secure: {
    group: 'chassis',
    type: 'boolean',
    description: 'enable security features via "helmet".  slight performance hit, but its recommended if this service is publically accessible',
    default: defaults.secure
  },

  cors: {
    group: 'chassis',
    type: 'boolean',
    description: 'enable cors for incoming requests',
    default: defaults.cors
  },

  compression: {
    group: 'chassis',
    type: 'boolean',
    description: 'gzip responses.  Recommended for mobile apps',
    default: defaults.compression
  },

  notfound: {
    group: 'chassis',
    type: 'boolean',
    description: 'handle not found errors and send 404 response',
    default: defaults.notfound
  },

  errors: {
    group: 'chassis',
    type: 'boolean',
    description: 'handle general errors and send apppropiate response',
    default: defaults.errors
  },

  type: {
    group: 'service',
    type: 'string',
    oneOf: ['authenticator, consumer, store, processor, dispatcher, mutator, cache'],
    description: 'The type of service',
    default: defaults.type
  },

  paginate: {
    group: 'service',
    type: 'object',
    options: {
      default: {
        type: 'number',
        description: 'The default umber of documents/entities per response',
        default: defaults.paginate.default
      },
      max: {
        type: 'number',
        description: 'The max umber of documents/entities per response a user can request',
        default: defaults.paginate.default
      }
    }
  },

  sockets: {
    group: 'service',
    type: 'boolean',
    description: 'enable socket connections to this service',
    default: defaults.sockets
  },


  authentication: {
    group: 'authentication',
    type: 'object',
    options: {
      strategies: {
        type: 'array',
        description: 'Authentications strategies',
        default: defaults.authentication.strategies
      },
      path: {
        type: 'string',
        description: 'the authentication path or endpoint',
        default: defaults.authentication.path
      },
      secret: {
        type: 'string',
        description: 'the secret used to generate and validaote JWTs',
        default: defaults.authentication.secret
      },
      // jwt: {
      //   type: 'object',
      //   options: {

      //   }
      // }
    }
  },
  docs: {
    group: 'documentation',
    type: 'boolean',
    description: 'Enable documentation',
    default: defaults.docs
  },

  docsEndpoint: {
    group: 'documentation',
    type: 'string',
    description: 'the path or endpoint at which documentation can be found',
    default: defaults.docsEndpoint
  },

  ping: {
    group: 'service',
    type: 'boolean',
    description: 'Enable Pinging',
    default: defaults.ping
  },

  pingEndpoint: {
    group: 'service',
    type: 'string',
    description: 'the path or endpoint at which ping data can be found',
    default: defaults.pingEndpoint
  },

  info: {
    group: 'service',
    type: 'boolean',
    description: 'Enable infoing',
    default: defaults.info
  },

  infoEndpoint: {
    group: 'service',
    type: 'string',
    description: 'the path or endpoint at which info data can be found',
    default: defaults.infoEndpoint
  },

  monitoring: {
    group: 'service',
    type: 'boolean',
    description: 'Enable monitoring',
    default: defaults.monitoring
  },

  monitoringEndpoint: {
    group: 'service',
    type: 'string',
    description: 'the path or endpoint at which monitoring data can be found',
    default: defaults.monitoringEndpoint
  },

  health: {
    group: 'service',
    type: 'boolean',
    description: 'Enable health',
    default: defaults.health
  },

  healthEndpoint: {
    group: 'service',
    type: 'string',
    description: 'the path or endpoint at which health data can be found',
    default: defaults.healthEndpoint
  }
}
